@@ -1,4 +1,4 @@
-"""Unstructured document chunker - extracted from partition-api."""
+"""Unstructured document chunker"""
 
 from io import BytesIO
 from pathlib import Path
@@ -15,8 +15,7 @@ class UnstructuredChunker(BaseChunker):
     """
     Document chunker using Unstructured.io library via LlamaIndex.
 
-    This implementation extracts the core logic from partition-api (main.py)
-    and integrates it directly into Maktaba for better performance and simplicity.
+    This implementation integrates Unstructured document chunking directly into Maktaba for better performance and simplicity.
 
     Supports:
     - Multiple file formats (PDF, DOCX, TXT, HTML, etc.)
@@ -109,10 +108,17 @@ class UnstructuredChunker(BaseChunker):
 
             # Add advanced chunking parameters if set
             overlap = kwargs.get("overlap", self.overlap)
+            max_chars = kwargs.get("max_characters", self.max_characters) or self.max_characters
+
             if overlap is not None:
                 unstructured_args["overlap"] = overlap
+                # overlap_all=True ensures overlap applies between ALL chunks, not just split chunks
+                # Only use it if overlap is reasonable (< 50% of max chunk size) to avoid duplicate chunks
+                # when chunks end up being smaller than expected
+                max_chars_for_check = max_chars or 1000
+                if overlap < max_chars_for_check * 0.5:
+                    unstructured_args["overlap_all"] = True
 
-            max_chars = kwargs.get("max_characters", self.max_characters)
             if max_chars is not None:
                 unstructured_args["max_characters"] = max_chars
 
@@ -213,10 +219,17 @@ class UnstructuredChunker(BaseChunker):
 
             # Add advanced chunking parameters if set
             overlap = kwargs.get("overlap", self.overlap)
+            max_chars = kwargs.get("max_characters", self.max_characters) or self.max_characters
+
             if overlap is not None:
                 unstructured_args["overlap"] = overlap
+                # overlap_all=True ensures overlap applies between ALL chunks, not just split chunks
+                # Only use it if overlap is reasonable (< 50% of max chunk size) to avoid duplicate chunks
+                # when chunks end up being smaller than expected
+                max_chars_for_check = max_chars or 1000
+                if overlap < max_chars_for_check * 0.5:
+                    unstructured_args["overlap_all"] = True
 
-            max_chars = kwargs.get("max_characters", self.max_characters)
             if max_chars is not None:
                 unstructured_args["max_characters"] = max_chars
 
@@ -322,10 +335,17 @@ class UnstructuredChunker(BaseChunker):
 
             # Add advanced chunking parameters if set
             overlap = kwargs.get("overlap", self.overlap)
+            max_chars = kwargs.get("max_characters", self.max_characters) or self.max_characters
+
             if overlap is not None:
                 unstructured_args["overlap"] = overlap
+                # overlap_all=True ensures overlap applies between ALL chunks, not just split chunks
+                # Only use it if overlap is reasonable (< 50% of max chunk size) to avoid duplicate chunks
+                # when chunks end up being smaller than expected
+                max_chars_for_check = max_chars or 1000
+                if overlap < max_chars_for_check * 0.5:
+                    unstructured_args["overlap_all"] = True
 
-            max_chars = kwargs.get("max_characters", self.max_characters)
             if max_chars is not None:
                 unstructured_args["max_characters"] = max_chars
 
@@ -381,7 +401,7 @@ class UnstructuredChunker(BaseChunker):
         """
         Extract total page count from documents.
 
-        Matches partition-api logic: finds max page_number in metadata.
+        # Finds the highest page number in document metadata.
         """
         total_pages = None
 
